@@ -4,6 +4,7 @@ import os
 import shlex
 import subprocess
 import tempfile
+import time
 
 ConfigDmgtools = namedtuple('Config', 'dmg hfsplus newfs_hfs verbose')
 ConfigHdiutil = namedtuple('Config', 'hdiutil verbose')
@@ -36,10 +37,10 @@ class Dmgtools(Dmg):
 	def _dmg(self, *args):
 		self._check_call((self.config.dmg,) + args)
 
-	def _create_hfs(self, hfs, volume_name, size):
+	def _create_hfs(self, hfs_fd, hfs, volume_name, size):
 		if self.config.verbose >= 1:
 			print("TRUNCATING {} to {} bytes".format(hfs, size))
-		with open(hfs, 'wb') as f:
+		with os.fdopen(hfs_fd, 'wb') as f:
 			f.truncate(size)
 		self._mkfs_hfs('-v', volume_name, hfs)
 
@@ -55,8 +56,8 @@ class Dmgtools(Dmg):
 	def create(self, dmg, volume_name, directory, symlinks):
 		input_size = sum(os.stat(os.path.join(path, f)).st_size for path, dirs, files in os.walk(directory) for f in files)
 		output_size = max(input_size * 2, 1024**2)
-		hfs = tempfile.mktemp(prefix=dmg + '.', suffix='.hfs')
-		self._create_hfs(hfs, volume_name, output_size)
+		hfs_fd, hfs = tempfile.mkstemp(prefix=dmg + '.', suffix='.hfs')
+		self._create_hfs(hfs_fd, hfs, volume_name, output_size)
 		self._add(hfs, directory)
 		for target, link_name in symlinks:
 			self._symlink(hfs, target, link_name)
@@ -72,7 +73,16 @@ class Hdiutil(Dmg):
 	def create(self, dmg, volume_name, directory, symlinks):
 		if symlinks:
 			raise NotImplementedError("symlinks are not yet implemented")
-		self._hdiutil('create', '-volname', volume_name, '-srcdir', directory, dmg)
+		for i in range(5):
+			try:
+				self._hdiutil('create', '-volname', volume_name, '-srcdir', directory, dmg)
+			except subprocess.CalledProcessError as e:
+				if i == 4:
+					raise e
+				print("Retrying hdiutil create")
+				time.sleep(5)
+			else:
+				break
 
 def main():
 	p = argparse.ArgumentParser(description="Manipulate dmg archives")
