@@ -51,6 +51,7 @@
 #define CONF_FAMILY_STRING "unix"
 #define CONF_PLATFORM_LINUX 1
 #define PLATFORM_STRING "linux"
+#define CONF_BACKEND_OPENGL_ES3 1
 #endif
 
 #if defined(__GNU__) || defined(__gnu__)
@@ -63,8 +64,8 @@
 #if defined(MACOSX) || defined(__APPLE__) || defined(__DARWIN__)
 #define CONF_FAMILY_UNIX 1
 #define CONF_FAMILY_STRING "unix"
-#define CONF_PLATFORM_MACOSX 1
-#define PLATFORM_STRING "macosx"
+#define CONF_PLATFORM_MACOS 1
+#define PLATFORM_STRING "macos"
 #endif
 
 #if defined(__sun)
@@ -82,6 +83,13 @@
 #define PLATFORM_STRING "beos"
 #endif
 
+#if defined(__HAIKU__)
+#define CONF_FAMILY_UNIX 1
+#define CONF_FAMILY_STRING "unix"
+#define CONF_PLATFORM_HAIKU 1
+#define CONF_PLATFORM_STRING "haiku"
+#endif
+
 /* use gcc endianness definitions when available */
 #if defined(__GNUC__) && !defined(__APPLE__) && !defined(__MINGW32__) && !defined(__sun)
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
