@@ -140,6 +140,7 @@ int CRequest::RunImpl(CURL *pHandle)
 	curl_easy_setopt(pHandle, CURLOPT_URL, m_aUrl);
 	curl_easy_setopt(pHandle, CURLOPT_NOSIGNAL, 1L);
 	curl_easy_setopt(pHandle, CURLOPT_USERAGENT, GAME_NAME " " GAME_RELEASE_VERSION " (" CONF_PLATFORM_STRING "; " CONF_ARCH_STRING ")");
+	curl_easy_setopt(pHandle, CURLOPT_ACCEPT_ENCODING, ""); // Use any compression algorithm supported by libcurl.
 
 	curl_easy_setopt(pHandle, CURLOPT_WRITEDATA, this);
 	curl_easy_setopt(pHandle, CURLOPT_WRITEFUNCTION, WriteCallback);
@@ -185,8 +186,24 @@ int CRequest::ProgressCallback(void *pUser, double DlTotal, double DlCurr, doubl
 	return pTask->m_Abort ? -1 : 0;
 }
 
-CGet::CGet(const char *pUrl, CTimeout Timeout) :
-	CRequest(pUrl, Timeout),
+CHead::CHead(const char *pUrl, CTimeout Timeout, bool LogProgress) :
+	CRequest(pUrl, Timeout, LogProgress)
+{
+}
+
+CHead::~CHead()
+{
+}
+
+bool CHead::AfterInit(void *pCurl)
+{
+	CURL *pHandle = pCurl;
+	curl_easy_setopt(pHandle, CURLOPT_NOBODY, 1L);
+	return true;
+}
+
+CGet::CGet(const char *pUrl, CTimeout Timeout, bool LogProgress) :
+	CRequest(pUrl, Timeout, LogProgress),
 	m_BufferSize(0),
 	m_BufferLength(0),
 	m_pBuffer(NULL)
