@@ -89,8 +89,6 @@ void CInput::MouseModeRelative()
 	m_InputGrabbed = 1;
 	SDL_SetRelativeMouseMode(SDL_TRUE);
 	Graphics()->SetWindowGrab(true);
-	// We should do this call to reset relative mouse position after alt+tab
-	SDL_GetRelativeMouseState(0x0, 0x0);
 }
 
 int CInput::MouseDoubleClick()
@@ -336,6 +334,8 @@ int CInput::Update()
 						MouseModeRelative();
 					m_MouseFocus = true;
 					IgnoreKeys = true;
+					// We should do this call to reset relative mouse position after alt+tab
+					SDL_GetRelativeMouseState(0x0, 0x0);
 					break;
 				case SDL_WINDOWEVENT_FOCUS_LOST:
 					m_MouseFocus = false;
@@ -347,7 +347,7 @@ int CInput::Update()
 						m_InputGrabbed = true;
 					}
 					break;
-#if defined(CONF_PLATFORM_MACOSX) // Todo: remove this when fixed in SDL
+#if defined(CONF_PLATFORM_MACOS) // Todo: remove this when fixed in SDL
 				case SDL_WINDOWEVENT_MAXIMIZED:
 					MouseModeAbsolute();
 					MouseModeRelative();
