@@ -1,21 +1,9 @@
 #ifndef ENGINE_CLIENT_VIDEO_H
 #define ENGINE_CLIENT_VIDEO_H
 
-#if defined(__ANDROID__)
-#define GL_GLEXT_PROTOTYPES
-#include <GL/glu.h>
-#include <GLES/gl.h>
-#include <GLES/glext.h>
-#define glOrtho glOrthof
-#else
-#include "SDL_opengl.h"
+#include <base/system.h>
 
-#if defined(CONF_PLATFORM_MACOSX)
-#include "OpenGL/glu.h"
-#else
-#include "GL/glu.h"
-#endif
-#endif
+#include "graphics_defines.h"
 
 extern "C" {
 #include <libavcodec/avcodec.h>
@@ -26,8 +14,6 @@ extern "C" {
 #include <libswscale/swscale.h>
 };
 
-#include <base/system.h>
-
 #include <engine/shared/demo.h>
 #include <engine/shared/video.h>
 #define ALEN 2048
@@ -116,7 +102,7 @@ private:
 
 	bool m_HasAudio;
 
-	GLubyte *m_pPixels;
+	TWGLubyte *m_pPixels;
 
 	OutputStream m_VideoStream;
 	OutputStream m_AudioStream;
