@@ -38,19 +38,17 @@ public:
 	// Get Median Map Time from l.Map
 	virtual const char *MedianMapTime(char *pBuffer, int BufferSize) const = 0;
 
-	enum Status
-	{
-		IN_USE,
-		SUCCESS,
-		FAILURE,
-	};
 	// tries to allocate the connection from the pool established
-	virtual Status Connect() = 0;
+	//
+	// returns true on failure
+	virtual bool Connect(char *pError, int ErrorSize) = 0;
 	// has to be called to return the connection back to the pool
 	virtual void Disconnect() = 0;
 
 	// ? for Placeholders, connection has to be established, can overwrite previous prepared statements
-	virtual void PrepareStatement(const char *pStmt) = 0;
+	//
+	// returns true on failure
+	virtual bool PrepareStatement(const char *pStmt, char *pError, int ErrorSize) = 0;
 
 	// PrepareStatement has to be called beforehand,
 	virtual void BindString(int Idx, const char *pString) = 0;
@@ -60,23 +58,27 @@ public:
 
 	// Print expanded sql statement
 	virtual void Print() = 0;
+
 	// executes the query and returns if a result row exists and selects it
 	// when called multiple times the next row is selected
-	virtual bool Step() = 0;
+	//
+	// returns true on failure
+	virtual bool Step(bool *pEnd, char *pError, int ErrorSize) = 0;
 	// executes the query and returns the number of rows affected by the update/insert/delete
-	// FIXME(2020-01-20): change function to AffectedRows() when moved to c-api of MySQL
-	virtual int ExecuteUpdate() = 0;
+	//
+	// returns true on failure
+	virtual bool ExecuteUpdate(int *pNumUpdated, char *pError, int ErrorSize) = 0;
 
-	virtual bool IsNull(int Col) const = 0;
-	virtual float GetFloat(int Col) const = 0;
-	virtual int GetInt(int Col) const = 0;
+	virtual bool IsNull(int Col) = 0;
+	virtual float GetFloat(int Col) = 0;
+	virtual int GetInt(int Col) = 0;
 	// ensures that the string is null terminated
-	virtual void GetString(int Col, char *pBuffer, int BufferSize) const = 0;
+	virtual void GetString(int Col, char *pBuffer, int BufferSize) = 0;
 	// returns number of bytes read into the buffer
-	virtual int GetBlob(int Col, unsigned char *pBuffer, int BufferSize) const = 0;
+	virtual int GetBlob(int Col, unsigned char *pBuffer, int BufferSize) = 0;
 
 	// SQL statements, that can't be abstracted, has side effects to the result
-	virtual void AddPoints(const char *pPlayer, int Points) = 0;
+	virtual bool AddPoints(const char *pPlayer, int Points, char *pError, int ErrorSize) = 0;
 
 private:
 	char m_aPrefix[64];
@@ -89,4 +91,18 @@ protected:
 	void FormatCreatePoints(char *aBuf, unsigned int BufferSize);
 };
 
+int MysqlInit();
+void MysqlUninit();
+
+IDbConnection *CreateSqliteConnection(const char *pFilename, bool Setup);
+// Returns nullptr if MySQL support is not compiled in.
+IDbConnection *CreateMysqlConnection(
+	const char *pDatabase,
+	const char *pPrefix,
+	const char *pUser,
+	const char *pPass,
+	const char *pIp,
+	int Port,
+	bool Setup);
+
 #endif // ENGINE_SERVER_DATABASES_CONNECTION_H
