@@ -1,10 +1,69 @@
-#include "sqlite.h"
+#include "connection.h"
+
+#include <sqlite3.h>
 
 #include <base/math.h>
 #include <engine/console.h>
 
-#include <sqlite3.h>
-#include <stdexcept>
+#include <atomic>
+
+class CSqliteConnection : public IDbConnection
+{
+public:
+	CSqliteConnection(const char *pFilename, bool Setup);
+	virtual ~CSqliteConnection();
+	virtual void Print(IConsole *pConsole, const char *Mode);
+
+	virtual CSqliteConnection *Copy();
+
+	virtual const char *BinaryCollate() const { return "BINARY"; }
+	virtual void ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize);
+	virtual const char *InsertTimestampAsUtc() const { return "DATETIME(?, 'utc')"; }
+	virtual const char *CollateNocase() const { return "? COLLATE NOCASE"; }
+	virtual const char *InsertIgnore() const { return "INSERT OR IGNORE"; };
+	virtual const char *Random() const { return "RANDOM()"; };
+	virtual const char *MedianMapTime(char *pBuffer, int BufferSize) const;
+
+	virtual bool Connect(char *pError, int ErrorSize);
+	virtual void Disconnect();
+
+	virtual bool PrepareStatement(const char *pStmt, char *pError, int ErrorSize);
+
+	virtual void BindString(int Idx, const char *pString);
+	virtual void BindBlob(int Idx, unsigned char *pBlob, int Size);
+	virtual void BindInt(int Idx, int Value);
+	virtual void BindFloat(int Idx, float Value);
+
+	virtual void Print();
+	virtual bool Step(bool *pEnd, char *pError, int ErrorSize);
+	virtual bool ExecuteUpdate(int *pNumUpdated, char *pError, int ErrorSize);
+
+	virtual bool IsNull(int Col);
+	virtual float GetFloat(int Col);
+	virtual int GetInt(int Col);
+	virtual void GetString(int Col, char *pBuffer, int BufferSize);
+	// passing a negative buffer size is undefined behavior
+	virtual int GetBlob(int Col, unsigned char *pBuffer, int BufferSize);
+
+	virtual bool AddPoints(const char *pPlayer, int Points, char *pError, int ErrorSize);
+
+private:
+	// copy of config vars
+	char m_aFilename[512];
+	bool m_Setup;
+
+	sqlite3 *m_pDb;
+	sqlite3_stmt *m_pStmt;
+	bool m_Done; // no more rows available for Step
+	// returns false, if the query succeeded
+	bool Execute(const char *pQuery, char *pError, int ErrorSize);
+
+	// returns true if an error was formatted
+	bool FormatError(int Result, char *pError, int ErrorSize);
+	void AssertNoError(int Result);
+
+	std::atomic_bool m_InUse;
+};
 
 CSqliteConnection::CSqliteConnection(const char *pFilename, bool Setup) :
 	IDbConnection("record"),
@@ -44,19 +103,23 @@ CSqliteConnection *CSqliteConnection::Copy()
 	return new CSqliteConnection(m_aFilename, m_Setup);
 }
 
-IDbConnection::Status CSqliteConnection::Connect()
+bool CSqliteConnection::Connect(char *pError, int ErrorSize)
 {
 	if(m_InUse.exchange(true))
-		return Status::IN_USE;
+	{
+		dbg_assert(0, "Tried connecting while the connection is in use");
+	}
 
 	if(m_pDb != nullptr)
-		return Status::SUCCESS;
+	{
+		return false;
+	}
 
 	int Result = sqlite3_open(m_aFilename, &m_pDb);
 	if(Result != SQLITE_OK)
 	{
-		dbg_msg("sql", "Can't open sqlite database: '%s'", sqlite3_errmsg(m_pDb));
-		return Status::FAILURE;
+		str_format(pError, ErrorSize, "Can't open sqlite database: '%s'", sqlite3_errmsg(m_pDb));
+		return true;
 	}
 
 	// wait for database to unlock so we don't have to handle SQLITE_BUSY errors
@@ -66,23 +129,23 @@ IDbConnection::Status CSqliteConnection::Connect()
 	{
 		char aBuf[1024];
 		FormatCreateRace(aBuf, sizeof(aBuf));
-		if(!Execute(aBuf))
-			return Status::FAILURE;
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
 		FormatCreateTeamrace(aBuf, sizeof(aBuf), "BLOB");
-		if(!Execute(aBuf))
-			return Status::FAILURE;
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
 		FormatCreateMaps(aBuf, sizeof(aBuf));
-		if(!Execute(aBuf))
-			return Status::FAILURE;
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
 		FormatCreateSaves(aBuf, sizeof(aBuf));
-		if(!Execute(aBuf))
-			return Status::FAILURE;
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
 		FormatCreatePoints(aBuf, sizeof(aBuf));
-		if(!Execute(aBuf))
-			return Status::FAILURE;
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
 		m_Setup = false;
 	}
-	return Status::SUCCESS;
+	return false;
 }
 
 void CSqliteConnection::Disconnect()
@@ -93,7 +156,7 @@ void CSqliteConnection::Disconnect()
 	m_InUse.store(false);
 }
 
-void CSqliteConnection::PrepareStatement(const char *pStmt)
+bool CSqliteConnection::PrepareStatement(const char *pStmt, char *pError, int ErrorSize)
 {
 	if(m_pStmt != nullptr)
 		sqlite3_finalize(m_pStmt);
@@ -104,35 +167,39 @@ void CSqliteConnection::PrepareStatement(const char *pStmt)
 		-1, // pStmt can be any length
 		&m_pStmt,
 		NULL);
-	ExceptionOnError(Result);
+	if(FormatError(Result, pError, ErrorSize))
+	{
+		return true;
+	}
 	m_Done = false;
+	return false;
 }
 
 void CSqliteConnection::BindString(int Idx, const char *pString)
 {
 	int Result = sqlite3_bind_text(m_pStmt, Idx, pString, -1, NULL);
-	ExceptionOnError(Result);
+	AssertNoError(Result);
 	m_Done = false;
 }
 
 void CSqliteConnection::BindBlob(int Idx, unsigned char *pBlob, int Size)
 {
 	int Result = sqlite3_bind_blob(m_pStmt, Idx, pBlob, Size, NULL);
-	ExceptionOnError(Result);
+	AssertNoError(Result);
 	m_Done = false;
 }
 
 void CSqliteConnection::BindInt(int Idx, int Value)
 {
 	int Result = sqlite3_bind_int(m_pStmt, Idx, Value);
-	ExceptionOnError(Result);
+	AssertNoError(Result);
 	m_Done = false;
 }
 
 void CSqliteConnection::BindFloat(int Idx, float Value)
 {
 	int Result = sqlite3_bind_double(m_pStmt, Idx, (double)Value);
-	ExceptionOnError(Result);
+	AssertNoError(Result);
 	m_Done = false;
 }
 
@@ -151,54 +218,68 @@ void CSqliteConnection::Print()
 	}
 }
 
-bool CSqliteConnection::Step()
+bool CSqliteConnection::Step(bool *pEnd, char *pError, int ErrorSize)
 {
 	if(m_Done)
+	{
+		*pEnd = true;
 		return false;
+	}
 	int Result = sqlite3_step(m_pStmt);
 	if(Result == SQLITE_ROW)
 	{
-		return true;
+		*pEnd = false;
+		return false;
 	}
 	else if(Result == SQLITE_DONE)
 	{
 		m_Done = true;
+		*pEnd = true;
 		return false;
 	}
 	else
 	{
-		ExceptionOnError(Result);
+		if(FormatError(Result, pError, ErrorSize))
+		{
+			return true;
+		}
 	}
+	*pEnd = true;
 	return false;
 }
 
-int CSqliteConnection::ExecuteUpdate()
+bool CSqliteConnection::ExecuteUpdate(int *pNumUpdated, char *pError, int ErrorSize)
 {
-	Step();
-	return sqlite3_changes(m_pDb);
+	bool End;
+	if(Step(&End, pError, ErrorSize))
+	{
+		return true;
+	}
+	*pNumUpdated = sqlite3_changes(m_pDb);
+	return false;
 }
 
-bool CSqliteConnection::IsNull(int Col) const
+bool CSqliteConnection::IsNull(int Col)
 {
 	return sqlite3_column_type(m_pStmt, Col - 1) == SQLITE_NULL;
 }
 
-float CSqliteConnection::GetFloat(int Col) const
+float CSqliteConnection::GetFloat(int Col)
 {
 	return (float)sqlite3_column_double(m_pStmt, Col - 1);
 }
 
-int CSqliteConnection::GetInt(int Col) const
+int CSqliteConnection::GetInt(int Col)
 {
 	return sqlite3_column_int(m_pStmt, Col - 1);
 }
 
-void CSqliteConnection::GetString(int Col, char *pBuffer, int BufferSize) const
+void CSqliteConnection::GetString(int Col, char *pBuffer, int BufferSize)
 {
 	str_copy(pBuffer, (const char *)sqlite3_column_text(m_pStmt, Col - 1), BufferSize);
 }
 
-int CSqliteConnection::GetBlob(int Col, unsigned char *pBuffer, int BufferSize) const
+int CSqliteConnection::GetBlob(int Col, unsigned char *pBuffer, int BufferSize)
 {
 	int Size = sqlite3_column_bytes(m_pStmt, Col - 1);
 	Size = minimum(Size, BufferSize);
@@ -223,27 +304,40 @@ const char *CSqliteConnection::MedianMapTime(char *pBuffer, int BufferSize) cons
 	return pBuffer;
 }
 
-bool CSqliteConnection::Execute(const char *pQuery)
+bool CSqliteConnection::Execute(const char *pQuery, char *pError, int ErrorSize)
 {
 	char *pErrorMsg;
 	int Result = sqlite3_exec(m_pDb, pQuery, NULL, NULL, &pErrorMsg);
 	if(Result != SQLITE_OK)
 	{
-		dbg_msg("sql", "error executing query: '%s'", pErrorMsg);
+		str_format(pError, ErrorSize, "error executing query: '%s'", pErrorMsg);
 		sqlite3_free(pErrorMsg);
+		return true;
 	}
-	return Result == SQLITE_OK;
+	return false;
 }
 
-void CSqliteConnection::ExceptionOnError(int Result)
+bool CSqliteConnection::FormatError(int Result, char *pError, int ErrorSize)
 {
 	if(Result != SQLITE_OK)
 	{
-		throw std::runtime_error(sqlite3_errmsg(m_pDb));
+		str_copy(pError, sqlite3_errmsg(m_pDb), ErrorSize);
+		return true;
+	}
+	return false;
+}
+
+void CSqliteConnection::AssertNoError(int Result)
+{
+	char aBuf[128];
+	if(FormatError(Result, aBuf, sizeof(aBuf)))
+	{
+		dbg_msg("sqlite", "unexpected sqlite error: %s", aBuf);
+		dbg_assert(0, "sqlite error");
 	}
 }
 
-void CSqliteConnection::AddPoints(const char *pPlayer, int Points)
+bool CSqliteConnection::AddPoints(const char *pPlayer, int Points, char *pError, int ErrorSize)
 {
 	char aBuf[512];
 	str_format(aBuf, sizeof(aBuf),
@@ -251,9 +345,22 @@ void CSqliteConnection::AddPoints(const char *pPlayer, int Points)
 		"VALUES (?, ?) "
 		"ON CONFLICT(Name) DO UPDATE SET Points=Points+?;",
 		GetPrefix());
-	PrepareStatement(aBuf);
+	if(PrepareStatement(aBuf, pError, ErrorSize))
+	{
+		return true;
+	}
 	BindString(1, pPlayer);
 	BindInt(2, Points);
 	BindInt(3, Points);
-	Step();
+	bool End;
+	if(Step(&End, pError, ErrorSize))
+	{
+		return true;
+	}
+	return false;
+}
+
+IDbConnection *CreateSqliteConnection(const char *pFilename, bool Setup)
+{
+	return new CSqliteConnection(pFilename, Setup);
 }
