@@ -38,7 +38,7 @@ class CEcon
 	int m_PrintCBIndex;
 	int m_UserClientID;
 
-	static void SendLineCB(const char *pLine, void *pUserData, bool Highlighted);
+	static void SendLineCB(const char *pLine, void *pUserData, ColorRGBA PrintColor = {1, 1, 1, 1});
 	static void ConchainEconOutputLevelUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConLogout(IConsole::IResult *pResult, void *pUserData);
 
