@@ -455,8 +455,8 @@ public:
 	// error and state
 	int NetType() const { return m_Socket.type; }
 	int State();
-	int GotProblems();
-	const char *ErrorString();
+	int GotProblems() const;
+	const char *ErrorString() const;
 
 	bool SecurityTokenUnknown() { return m_Connection.SecurityToken() == NET_SECURITY_TOKEN_UNKNOWN; }
 };
