@@ -20,9 +20,11 @@ enum
 	UNPACKMESSAGE_OK,
 	UNPACKMESSAGE_ANSWER,
 
-	SERVERCAP_CURVERSION = 1,
+	SERVERCAP_CURVERSION = 3,
 	SERVERCAPFLAG_DDNET = 1 << 0,
 	SERVERCAPFLAG_CHATTIMEOUTCODE = 1 << 1,
+	SERVERCAPFLAG_ANYPLAYERFLAG = 1 << 2,
+	SERVERCAPFLAG_PINGEX = 1 << 3,
 };
 
 void RegisterUuids(class CUuidManager *pManager);
