@@ -13,9 +13,9 @@ public:
 	int m_TypeAndID;
 
 	int *Data() { return (int *)(this + 1); }
-	int Type() { return m_TypeAndID >> 16; }
-	int ID() { return m_TypeAndID & 0xffff; }
-	int Key() { return m_TypeAndID; }
+	int Type() const { return m_TypeAndID >> 16; }
+	int ID() const { return m_TypeAndID & 0xffff; }
+	int Key() const { return m_TypeAndID; }
 };
 
 class CSnapshot
@@ -42,10 +42,10 @@ public:
 		m_NumItems = 0;
 	}
 	int NumItems() const { return m_NumItems; }
-	CSnapshotItem *GetItem(int Index);
-	int GetItemSize(int Index);
-	int GetItemIndex(int Key);
-	int GetItemType(int Index);
+	CSnapshotItem *GetItem(int Index) const;
+	int GetItemSize(int Index) const;
+	int GetItemIndex(int Key) const;
+	int GetItemType(int Index) const;
 
 	unsigned Crc();
 	void DebugDump();
