@@ -6,6 +6,10 @@
 #include <engine/client/updater.h>
 #include <engine/storage.h>
 
+#ifdef CONF_PLATFORM_HAIKU
+#include <stdlib.h>
+#endif
+
 class CStorage : public IStorage
 {
 public:
@@ -193,6 +197,9 @@ public:
 
 		// 3) check for usable path in argv[0]
 		{
+#ifdef CONF_PLATFORM_HAIKU
+			pArgv0 = realpath(pArgv0, NULL);
+#endif
 			unsigned int Pos = ~0U;
 			for(unsigned i = 0; pArgv0[i]; i++)
 				if(pArgv0[i] == '/' || pArgv0[i] == '\\')
@@ -211,6 +218,9 @@ public:
 				}
 			}
 		}
+#ifdef CONF_PLATFORM_HAIKU
+		free((void *)pArgv0);
+#endif
 
 #if defined(CONF_FAMILY_UNIX)
 		// 4) check for all default locations
@@ -269,7 +279,7 @@ public:
 				}
 				else
 				{
-#if defined(CONF_PLATFORM_MACOSX)
+#if defined(CONF_PLATFORM_MACOS)
 					str_append(m_aBinarydir, "/../../../DDNet-Server.app/Contents/MacOS", sizeof(m_aBinarydir));
 					str_format(aBuf, sizeof(aBuf), "%s/" PLAT_SERVER_EXEC, m_aBinarydir);
 					IOHANDLE File = io_open(aBuf, IOFLAG_READ);
@@ -584,7 +594,10 @@ void IStorage::StripPathAndExtension(const char *pFilename, char *pBuffer, int B
 	str_copy(pBuffer, pExtractedName, Length);
 }
 
-IStorage *CreateStorage(const char *pApplicationName, int StorageType, int NumArgs, const char **ppArguments) { return CStorage::Create(pApplicationName, StorageType, NumArgs, ppArguments); }
+IStorage *CreateStorage(const char *pApplicationName, int StorageType, int NumArgs, const char **ppArguments)
+{
+	return CStorage::Create(pApplicationName, StorageType, NumArgs, ppArguments);
+}
 
 IStorage *CreateLocalStorage()
 {
@@ -600,3 +613,13 @@ IStorage *CreateLocalStorage()
 	}
 	return pStorage;
 }
+IStorage *CreateTempStorage(const char *pDirectory)
+{
+	CStorage *pStorage = new CStorage();
+	if(!pStorage)
+	{
+		return nullptr;
+	}
+	pStorage->AddPath(pDirectory);
+	return pStorage;
+}
