@@ -17,6 +17,8 @@
 #include <game/client/gameclient.h>
 #include <game/collision.h>
 
+#include <base/vmath.h>
+
 #include "controls.h"
 
 enum
@@ -198,10 +200,6 @@ void CControls::OnConsoleInit()
 		static CInputState s_State = {this, &m_ShowHookColl[0], &m_ShowHookColl[1]};
 		Console()->Register("+showhookcoll", "", CFGFLAG_CLIENT, ConKeyInputState, (void *)&s_State, "Show Hook Collision");
 	}
-	{
-		static CInputState s_State = {this, &m_ResetDummy[0], &m_ResetDummy[1]};
-		Console()->Register("+resetdummy", "", CFGFLAG_CLIENT, ConKeyInputState, (void *)&s_State, "Reset Dummy");
-	}
 
 	{
 		static CInputSet s_Set = {this, &m_InputData[0].m_WantedWeapon, &m_InputData[1].m_WantedWeapon, 1};
@@ -341,19 +339,6 @@ int CControls::SnapInput(int *pData)
 			pDummyInput->m_Hook = g_Config.m_ClDummyHook;
 		}
 
-		if(m_ResetDummy[g_Config.m_ClDummy])
-		{
-			ResetInput(!g_Config.m_ClDummy);
-			m_InputData[!g_Config.m_ClDummy].m_Hook = 0;
-
-			CNetObj_PlayerInput *pDummyInput = &m_pClient->m_DummyInput;
-			pDummyInput->m_Hook = m_InputData[!g_Config.m_ClDummy].m_Hook;
-			pDummyInput->m_Jump = m_InputData[!g_Config.m_ClDummy].m_Jump;
-			pDummyInput->m_Direction = m_InputData[!g_Config.m_ClDummy].m_Jump;
-
-			pDummyInput->m_Fire = m_InputData[!g_Config.m_ClDummy].m_Fire;
-		}
-
 		// stress testing
 #ifdef CONF_DEBUG
 		if(g_Config.m_DbgStress)
@@ -589,9 +574,17 @@ void CControls::ClampMousePos()
 		float MinDistance = g_Config.m_ClDyncam ? g_Config.m_ClDyncamMinDistance : g_Config.m_ClMouseMinDistance;
 		float MouseMin = MinDistance;
 
-		if(length(m_MousePos[g_Config.m_ClDummy]) < MouseMin)
-			m_MousePos[g_Config.m_ClDummy] = normalize(m_MousePos[g_Config.m_ClDummy]) * MouseMin;
-		if(length(m_MousePos[g_Config.m_ClDummy]) > MouseMax)
-			m_MousePos[g_Config.m_ClDummy] = normalize(m_MousePos[g_Config.m_ClDummy]) * MouseMax;
+		float MDistance = length(m_MousePos[g_Config.m_ClDummy]);
+		if(MDistance < 0.001f)
+		{
+			m_MousePos[g_Config.m_ClDummy].x = 0.001f;
+			m_MousePos[g_Config.m_ClDummy].y = 0;
+			MDistance = 0.001f;
+		}
+		if(MDistance < MouseMin)
+			m_MousePos[g_Config.m_ClDummy] = normalize_pre_length(m_MousePos[g_Config.m_ClDummy], MDistance) * MouseMin;
+		MDistance = length(m_MousePos[g_Config.m_ClDummy]);
+		if(MDistance > MouseMax)
+			m_MousePos[g_Config.m_ClDummy] = normalize_pre_length(m_MousePos[g_Config.m_ClDummy], MDistance) * MouseMax;
 	}
 }
