@@ -8,9 +8,9 @@
 #include <game/generated/protocol.h>
 
 #include <game/client/gameclient.h>
+#include <game/client/projectile_data.h>
 #include <game/client/render.h>
 #include <game/client/ui.h>
-#include <game/extrainfo.h>
 
 #include <game/client/components/effects.h>
 #include <game/client/components/flow.h>
@@ -22,27 +22,28 @@ void CItems::OnReset()
 	m_NumExtraProjectiles = 0;
 }
 
-void CItems::RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID)
+void CItems::RenderProjectile(const CProjectileData *pCurrent, int ItemID)
 {
 	int CurWeapon = clamp(pCurrent->m_Type, 0, NUM_WEAPONS - 1);
 
 	// get positions
 	float Curvature = 0;
 	float Speed = 0;
+	CTuningParams Tuning = GameClient()->GetTunes(pCurrent->m_TuneZone);
 	if(CurWeapon == WEAPON_GRENADE)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeSpeed;
+		Curvature = Tuning.m_GrenadeCurvature;
+		Speed = Tuning.m_GrenadeSpeed;
 	}
 	else if(CurWeapon == WEAPON_SHOTGUN)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunSpeed;
+		Curvature = Tuning.m_ShotgunCurvature;
+		Speed = Tuning.m_ShotgunSpeed;
 	}
 	else if(CurWeapon == WEAPON_GUN)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunSpeed;
+		Curvature = Tuning.m_GunCurvature;
+		Speed = Tuning.m_GunSpeed;
 	}
 
 	bool LocalPlayerInGame = false;
@@ -54,29 +55,23 @@ void CItems::RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID)
 	if(m_pClient->m_Snap.m_pGameInfoObj && !(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags & GAMESTATEFLAG_PAUSED))
 		s_LastGameTickTime = Client()->GameTickTime(g_Config.m_ClDummy);
 
+	bool IsOtherTeam = (pCurrent->m_ExtraInfo && pCurrent->m_Owner >= 0 && m_pClient->IsOtherTeam(pCurrent->m_Owner));
+
 	float Ct;
-	if(m_pClient->Predict() && m_pClient->AntiPingGrenade() && LocalPlayerInGame && !(Client()->State() == IClient::STATE_DEMOPLAYBACK))
+	if(m_pClient->Predict() && m_pClient->AntiPingGrenade() && LocalPlayerInGame && !IsOtherTeam)
 		Ct = ((float)(Client()->PredGameTick(g_Config.m_ClDummy) - 1 - pCurrent->m_StartTick) + Client()->PredIntraGameTick(g_Config.m_ClDummy)) / (float)SERVER_TICK_SPEED;
 	else
 		Ct = (Client()->PrevGameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) / (float)SERVER_TICK_SPEED + s_LastGameTickTime;
 	if(Ct < 0)
 		return; // projectile haven't been shot yet
 
-	vec2 StartPos;
-	vec2 StartVel;
-
-	ExtractInfo(pCurrent, &StartPos, &StartVel);
-
-	vec2 Pos = CalcPos(StartPos, StartVel, Curvature, Speed, Ct);
-	vec2 PrevPos = CalcPos(StartPos, StartVel, Curvature, Speed, Ct - 0.001f);
+	vec2 Pos = CalcPos(pCurrent->m_StartPos, pCurrent->m_StartVel, Curvature, Speed, Ct);
+	vec2 PrevPos = CalcPos(pCurrent->m_StartPos, pCurrent->m_StartVel, Curvature, Speed, Ct - 0.001f);
 
 	float Alpha = 1.f;
-	if(UseExtraInfo(pCurrent))
+	if(IsOtherTeam)
 	{
-		int Owner;
-		ExtractExtraInfo(pCurrent, &Owner, 0, 0, 0);
-		if(Owner >= 0 && m_pClient->IsOtherTeam(Owner))
-			Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
+		Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
 	}
 
 	vec2 Vel = Pos - PrevPos;
@@ -241,6 +236,8 @@ void CItems::RenderLaser(const struct CNetObj_Laser *pCurrent, bool IsPredicted)
 	RGB = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClLaserInnerColor));
 	ColorRGBA InnerColor(RGB.r, RGB.g, RGB.b, 1.0f);
 
+	int TuneZone = GameClient()->m_GameWorld.m_WorldConfig.m_UseTuneZones ? Collision()->IsTune(Collision()->GetMapIndex(From)) : 0;
+
 	vec2 Dir;
 	if(Len > 0)
 	{
@@ -252,7 +249,7 @@ void CItems::RenderLaser(const struct CNetObj_Laser *pCurrent, bool IsPredicted)
 		else
 			Ticks = (float)(Client()->GameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->IntraGameTick(g_Config.m_ClDummy);
 		float Ms = (Ticks / 50.0f) * 1000.0f;
-		float a = Ms / m_pClient->m_Tuning[g_Config.m_ClDummy].m_LaserBounceDelay;
+		float a = Ms / m_pClient->GetTunes(TuneZone).m_LaserBounceDelay;
 		a = clamp(a, 0.0f, 1.0f);
 		float Ia = 1 - a;
 
@@ -310,11 +307,7 @@ void CItems::OnRender()
 	{
 		for(auto *pProj = (CProjectile *)GameClient()->m_PredictedWorld.FindFirst(CGameWorld::ENTTYPE_PROJECTILE); pProj; pProj = (CProjectile *)pProj->NextEntity())
 		{
-			CNetObj_Projectile Data;
-			if(pProj->m_Type != WEAPON_SHOTGUN || pProj->m_Explosive || pProj->m_Freeze)
-				pProj->FillExtraInfo(&Data);
-			else
-				pProj->FillInfo(&Data);
+			CProjectileData Data = pProj->GetData();
 			RenderProjectile(&Data, pProj->ID());
 		}
 		for(auto *pLaser = (CLaser *)GameClient()->m_PredictedWorld.FindFirst(CGameWorld::ENTTYPE_LASER); pLaser; pLaser = (CLaser *)pLaser->NextEntity())
@@ -327,12 +320,15 @@ void CItems::OnRender()
 		}
 		for(auto *pPickup = (CPickup *)GameClient()->m_PredictedWorld.FindFirst(CGameWorld::ENTTYPE_PICKUP); pPickup; pPickup = (CPickup *)pPickup->NextEntity())
 		{
-			if(auto *pPrev = (CPickup *)GameClient()->m_PrevPredictedWorld.GetEntity(pPickup->ID(), CGameWorld::ENTTYPE_PICKUP))
+			if(pPickup->InDDNetTile())
 			{
-				CNetObj_Pickup Data, Prev;
-				pPickup->FillInfo(&Data);
-				pPrev->FillInfo(&Prev);
-				RenderPickup(&Prev, &Data, true);
+				if(auto *pPrev = (CPickup *)GameClient()->m_PrevPredictedWorld.GetEntity(pPickup->ID(), CGameWorld::ENTTYPE_PICKUP))
+				{
+					CNetObj_Pickup Data, Prev;
+					pPickup->FillInfo(&Data);
+					pPrev->FillInfo(&Prev);
+					RenderPickup(&Prev, &Data, true);
+				}
 			}
 		}
 	}
@@ -343,29 +339,44 @@ void CItems::OnRender()
 		IClient::CSnapItem Item;
 		const void *pData = Client()->SnapGetItem(IClient::SNAP_CURRENT, i, &Item);
 
-		if(Item.m_Type == NETOBJTYPE_PROJECTILE)
+		if(Item.m_Type == NETOBJTYPE_PROJECTILE || Item.m_Type == NETOBJTYPE_DDNETPROJECTILE)
 		{
+			CProjectileData Data;
+			if(Item.m_Type == NETOBJTYPE_PROJECTILE)
+			{
+				Data = ExtractProjectileInfo((const CNetObj_Projectile *)pData, &GameClient()->m_GameWorld);
+			}
+			else
+			{
+				Data = ExtractProjectileInfoDDNet((const CNetObj_DDNetProjectile *)pData, &GameClient()->m_GameWorld);
+			}
 			if(UsePredicted)
 			{
 				if(auto *pProj = (CProjectile *)GameClient()->m_GameWorld.FindMatch(Item.m_ID, Item.m_Type, pData))
 				{
+					bool IsOtherTeam = m_pClient->IsOtherTeam(pProj->GetOwner());
 					if(pProj->m_LastRenderTick <= 0 && (pProj->m_Type != WEAPON_SHOTGUN || (!pProj->m_Freeze && !pProj->m_Explosive)) // skip ddrace shotgun bullets
 						&& (pProj->m_Type == WEAPON_SHOTGUN || fabs(length(pProj->m_Direction) - 1.f) < 0.02) // workaround to skip grenades on ball mod
-						&& (pProj->GetOwner() < 0 || !GameClient()->m_aClients[pProj->GetOwner()].m_IsPredictedLocal) // skip locally predicted projectiles
+						&& (pProj->GetOwner() < 0 || !GameClient()->m_aClients[pProj->GetOwner()].m_IsPredictedLocal || IsOtherTeam) // skip locally predicted projectiles
 						&& !Client()->SnapFindItem(IClient::SNAP_PREV, Item.m_Type, Item.m_ID))
 					{
-						ReconstructSmokeTrail((const CNetObj_Projectile *)pData, Item.m_ID, pProj->m_DestroyTick);
+						ReconstructSmokeTrail(&Data, pProj->m_DestroyTick);
 					}
 					pProj->m_LastRenderTick = Client()->GameTick(g_Config.m_ClDummy);
-					continue;
+					if(!IsOtherTeam)
+						continue;
 				}
 			}
-			RenderProjectile((const CNetObj_Projectile *)pData, Item.m_ID);
+			RenderProjectile(&Data, Item.m_ID);
 		}
 		else if(Item.m_Type == NETOBJTYPE_PICKUP)
 		{
 			if(UsePredicted)
-				continue;
+			{
+				auto *pPickup = (CPickup *)GameClient()->m_GameWorld.FindMatch(Item.m_ID, Item.m_Type, pData);
+				if(pPickup && pPickup->InDDNetTile())
+					continue;
+			}
 			const void *pPrev = Client()->SnapFindItem(IClient::SNAP_PREV, Item.m_Type, Item.m_ID);
 			if(pPrev)
 				RenderPickup((const CNetObj_Pickup *)pPrev, (const CNetObj_Pickup *)pData);
@@ -409,7 +420,10 @@ void CItems::OnRender()
 			m_NumExtraProjectiles--;
 		}
 		else if(!UsePredicted)
-			RenderProjectile(&m_aExtraProjectiles[i], 0);
+		{
+			CProjectileData Data = ExtractProjectileInfo(&m_aExtraProjectiles[i], &GameClient()->m_GameWorld);
+			RenderProjectile(&Data, 0);
+		}
 	}
 
 	Graphics()->QuadsSetRotation(0);
@@ -484,7 +498,7 @@ void CItems::AddExtraProjectile(CNetObj_Projectile *pProj)
 	}
 }
 
-void CItems::ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemID, int DestroyTick)
+void CItems::ReconstructSmokeTrail(const CProjectileData *pCurrent, int DestroyTick)
 {
 	bool LocalPlayerInGame = false;
 
@@ -498,20 +512,22 @@ void CItems::ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemI
 	// get positions
 	float Curvature = 0;
 	float Speed = 0;
+	CTuningParams Tuning = GameClient()->GetTunes(pCurrent->m_TuneZone);
+
 	if(pCurrent->m_Type == WEAPON_GRENADE)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GrenadeSpeed;
+		Curvature = Tuning.m_GrenadeCurvature;
+		Speed = Tuning.m_GrenadeSpeed;
 	}
 	else if(pCurrent->m_Type == WEAPON_SHOTGUN)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_ShotgunSpeed;
+		Curvature = Tuning.m_ShotgunCurvature;
+		Speed = Tuning.m_ShotgunSpeed;
 	}
 	else if(pCurrent->m_Type == WEAPON_GUN)
 	{
-		Curvature = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunCurvature;
-		Speed = m_pClient->m_Tuning[g_Config.m_ClDummy].m_GunSpeed;
+		Curvature = Tuning.m_GunCurvature;
+		Speed = Tuning.m_GunSpeed;
 	}
 
 	float Pt = ((float)(Client()->PredGameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) + Client()->PredIntraGameTick(g_Config.m_ClDummy)) / (float)SERVER_TICK_SPEED;
@@ -520,18 +536,10 @@ void CItems::ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemI
 
 	float Gt = (Client()->PrevGameTick(g_Config.m_ClDummy) - pCurrent->m_StartTick) / (float)SERVER_TICK_SPEED + Client()->GameTickTime(g_Config.m_ClDummy);
 
-	vec2 StartPos;
-	vec2 StartVel;
-
-	ExtractInfo(pCurrent, &StartPos, &StartVel);
-
 	float Alpha = 1.f;
-	if(UseExtraInfo(pCurrent))
+	if(pCurrent->m_ExtraInfo && pCurrent->m_Owner >= 0 && m_pClient->IsOtherTeam(pCurrent->m_Owner))
 	{
-		int Owner;
-		ExtractExtraInfo(pCurrent, &Owner, 0, 0, 0);
-		if(Owner >= 0 && m_pClient->IsOtherTeam(Owner))
-			Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
+		Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
 	}
 
 	float T = Pt;
@@ -543,8 +551,8 @@ void CItems::ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemI
 	for(int i = 1 + (int)(Gt / Step); i < (int)(T / Step); i++)
 	{
 		float t = Step * (float)i + 0.4f * Step * (frandom() - 0.5f);
-		vec2 Pos = CalcPos(StartPos, StartVel, Curvature, Speed, t);
-		vec2 PrevPos = CalcPos(StartPos, StartVel, Curvature, Speed, t - 0.001f);
+		vec2 Pos = CalcPos(pCurrent->m_StartPos, pCurrent->m_StartVel, Curvature, Speed, t);
+		vec2 PrevPos = CalcPos(pCurrent->m_StartPos, pCurrent->m_StartVel, Curvature, Speed, t - 0.001f);
 		vec2 Vel = Pos - PrevPos;
 		float TimePassed = Pt - t;
 		if(Pt - MinTrailSpan > 0.01f)
