@@ -4,6 +4,8 @@
 #define GAME_CLIENT_COMPONENTS_ITEMS_H
 #include <game/client/component.h>
 
+class CProjectileData;
+
 class CItems : public CComponent
 {
 	enum
@@ -14,7 +16,7 @@ class CItems : public CComponent
 	CNetObj_Projectile m_aExtraProjectiles[MAX_EXTRA_PROJECTILES];
 	int m_NumExtraProjectiles;
 
-	void RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID);
+	void RenderProjectile(const CProjectileData *pCurrent, int ItemID);
 	void RenderPickup(const CNetObj_Pickup *pPrev, const CNetObj_Pickup *pCurrent, bool IsPredicted = false);
 	void RenderFlag(const CNetObj_Flag *pPrev, const CNetObj_Flag *pCurrent, const CNetObj_GameData *pPrevGameData, const CNetObj_GameData *pCurGameData);
 	void RenderLaser(const struct CNetObj_Laser *pCurrent, bool IsPredicted = false);
@@ -28,7 +30,7 @@ public:
 
 	void AddExtraProjectile(CNetObj_Projectile *pProj);
 
-	void ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemID, int DestroyTick);
+	void ReconstructSmokeTrail(const CProjectileData *pCurrent, int DestroyTick);
 };
 
 #endif
