@@ -238,6 +238,10 @@ public:
 	};
 
 	CSnapState m_Snap;
+	int m_LocalTuneZone[2];
+	bool m_ReceivedTuning[2];
+	int m_ExpectingTuningForZone[2];
+	int m_ExpectingTuningSince[2];
 
 	// client data
 	struct CClientData
@@ -383,11 +387,11 @@ public:
 
 	void OnLanguageChange();
 
-	virtual const char *GetItemName(int Type);
-	virtual const char *Version();
-	virtual const char *NetVersion();
-	virtual int DDNetVersion();
-	virtual const char *DDNetVersionStr();
+	virtual const char *GetItemName(int Type) const;
+	virtual const char *Version() const;
+	virtual const char *NetVersion() const;
+	virtual int DDNetVersion() const;
+	virtual const char *DDNetVersionStr() const;
 
 	// actions
 	// TODO: move these
@@ -446,11 +450,13 @@ public:
 	bool AntiPingGunfire() { return AntiPingGrenade() && AntiPingWeapons() && g_Config.m_ClAntiPingGunfire; }
 	bool Predict() { return g_Config.m_ClPredict && !(m_Snap.m_pGameInfoObj && m_Snap.m_pGameInfoObj->m_GameStateFlags & GAMESTATEFLAG_GAMEOVER) && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK && m_Snap.m_pLocalCharacter; }
 	bool PredictDummy() { return g_Config.m_ClPredictDummy && Client()->DummyConnected() && m_Snap.m_LocalClientID >= 0 && m_PredictedDummyID >= 0 && !m_aClients[m_PredictedDummyID].m_Paused; }
+	CTuningParams GetTunes(int i) { return m_aTuningList[i]; }
 
 	CGameWorld m_GameWorld;
 	CGameWorld m_PredictedWorld;
 	CGameWorld m_PrevPredictedWorld;
 
+	void DummyResetInput();
 	void Echo(const char *pString);
 	bool IsOtherTeam(int ClientID);
 	bool CanDisplayWarning();
