@@ -118,14 +118,14 @@ int CUI::Update(float Mx, float My, float Mwx, float Mwy, int Buttons)
 	return 0;
 }
 
-int CUI::MouseInside(const CUIRect *r)
+int CUI::MouseInside(const CUIRect *r) const
 {
 	if(m_MouseX >= r->x && m_MouseX < r->x + r->w && m_MouseY >= r->y && m_MouseY < r->y + r->h)
 		return 1;
 	return 0;
 }
 
-void CUI::ConvertMouseMove(float *x, float *y)
+void CUI::ConvertMouseMove(float *x, float *y) const
 {
 	float Fac = (float)(g_Config.m_UiMousesens) / g_Config.m_InpMousesens;
 	*x = *x * Fac;
@@ -156,7 +156,7 @@ void CUI::SetScale(float s)
 	g_Config.m_UiScale = (int)(s * 100.0f);
 }
 
-float CUI::Scale()
+float CUI::Scale() const
 {
 	return g_Config.m_UiScale / 100.0f;
 }
