@@ -61,8 +61,8 @@ public:
 	}
 	bool Set(int Index, float Value);
 	bool Set(const char *pName, float Value);
-	bool Get(int Index, float *pValue);
-	bool Get(const char *pName, float *pValue);
+	bool Get(int Index, float *pValue) const;
+	bool Get(const char *pName, float *pValue) const;
 };
 
 inline void StrToInts(int *pInts, int Num, const char *pStr)
@@ -265,6 +265,7 @@ public:
 	bool m_HasTelegunLaser;
 	int m_FreezeEnd;
 	bool m_DeepFrozen;
+	CTuningParams m_Tuning;
 
 private:
 	CTeamsCore *m_pTeams;
