@@ -5,6 +5,7 @@
 #include <game/server/gamecontext.h>
 #include <game/server/gamemodes/DDRace.h>
 #include <game/server/player.h>
+#include <game/version.h>
 
 #include <engine/shared/config.h>
 #include <game/server/teams.h>
@@ -49,7 +50,7 @@ CProjectile::CProjectile(
 void CProjectile::Reset()
 {
 	if(m_LifeSpan > -2)
-		GameServer()->m_World.DestroyEntity(this);
+		m_MarkedForDestroy = true;
 }
 
 vec2 CProjectile::GetPos(float Time)
@@ -143,7 +144,7 @@ void CProjectile::Tick()
 	}
 	else if(m_Owner >= 0 && (m_Type != WEAPON_GRENADE || g_Config.m_SvDestroyBulletsOnDeath))
 	{
-		GameServer()->m_World.DestroyEntity(this);
+		m_MarkedForDestroy = true;
 		return;
 	}
 
@@ -231,14 +232,14 @@ void CProjectile::Tick()
 		else if(m_Type == WEAPON_GUN)
 		{
 			GameServer()->CreateDamageInd(CurPos, -atan2(m_Direction.x, m_Direction.y), 10, (m_Owner != -1) ? TeamMask : -1LL);
-			GameServer()->m_World.DestroyEntity(this);
+			m_MarkedForDestroy = true;
 			return;
 		}
 		else
 		{
 			if(!m_Freeze)
 			{
-				GameServer()->m_World.DestroyEntity(this);
+				m_MarkedForDestroy = true;
 				return;
 			}
 		}
@@ -261,7 +262,7 @@ void CProjectile::Tick()
 			GameServer()->CreateSound(ColPos, m_SoundImpact,
 				(m_Owner != -1) ? TeamMask : -1LL);
 		}
-		GameServer()->m_World.DestroyEntity(this);
+		m_MarkedForDestroy = true;
 		return;
 	}
 
@@ -319,13 +320,27 @@ void CProjectile::Snap(int SnappingClient)
 	if(m_Owner != -1 && !CmaskIsSet(TeamMask, SnappingClient))
 		return;
 
-	CNetObj_Projectile *pProj = static_cast<CNetObj_Projectile *>(Server()->SnapNewItem(NETOBJTYPE_PROJECTILE, GetID(), sizeof(CNetObj_Projectile)));
-	if(pProj)
+	int SnappingClientVersion = SnappingClient >= 0 ? GameServer()->GetClientVersion(SnappingClient) : CLIENT_VERSIONNR;
+
+	CNetObj_DDNetProjectile DDNetProjectile;
+	if(SnappingClientVersion >= VERSION_DDNET_ANTIPING_PROJECTILE && FillExtraInfo(&DDNetProjectile))
 	{
-		if(SnappingClient > -1 && GameServer()->m_apPlayers[SnappingClient] && GameServer()->m_apPlayers[SnappingClient]->GetClientVersion() >= VERSION_DDNET_ANTIPING_PROJECTILE)
-			FillExtraInfo(pProj);
-		else
-			FillInfo(pProj);
+		int Type = SnappingClientVersion < VERSION_DDNET_MSG_LEGACY ? (int)NETOBJTYPE_PROJECTILE : NETOBJTYPE_DDNETPROJECTILE;
+		void *pProj = Server()->SnapNewItem(Type, GetID(), sizeof(DDNetProjectile));
+		if(!pProj)
+		{
+			return;
+		}
+		mem_copy(pProj, &DDNetProjectile, sizeof(DDNetProjectile));
+	}
+	else
+	{
+		CNetObj_Projectile *pProj = static_cast<CNetObj_Projectile *>(Server()->SnapNewItem(NETOBJTYPE_PROJECTILE, GetID(), sizeof(CNetObj_Projectile)));
+		if(!pProj)
+		{
+			return;
+		}
+		FillInfo(pProj);
 	}
 }
 
@@ -336,14 +351,13 @@ void CProjectile::SetBouncing(int Value)
 	m_Bouncing = Value;
 }
 
-void CProjectile::FillExtraInfo(CNetObj_Projectile *pProj)
+bool CProjectile::FillExtraInfo(CNetObj_DDNetProjectile *pProj)
 {
 	const int MaxPos = 0x7fffffff / 100;
 	if(abs((int)m_Pos.y) + 1 >= MaxPos || abs((int)m_Pos.x) + 1 >= MaxPos)
 	{
 		//If the modified data would be too large to fit in an integer, send normal data instead
-		FillInfo(pProj);
-		return;
+		return false;
 	}
 	//Send additional/modified info, by modifiying the fields of the netobj
 	float Angle = -atan2f(m_Direction.x, m_Direction.y);
@@ -351,18 +365,21 @@ void CProjectile::FillExtraInfo(CNetObj_Projectile *pProj)
 	int Data = 0;
 	Data |= (abs(m_Owner) & 255) << 0;
 	if(m_Owner < 0)
-		Data |= 1 << 8;
-	Data |= 1 << 9; //This bit tells the client to use the extra info
+		Data |= PROJECTILEFLAG_NO_OWNER;
+	//This bit tells the client to use the extra info
+	Data |= PROJECTILEFLAG_IS_DDNET;
+	// PROJECTILEFLAG_BOUNCE_HORIZONTAL, PROJECTILEFLAG_BOUNCE_VERTICAL
 	Data |= (m_Bouncing & 3) << 10;
 	if(m_Explosive)
-		Data |= 1 << 12;
+		Data |= PROJECTILEFLAG_EXPLOSIVE;
 	if(m_Freeze)
-		Data |= 1 << 13;
+		Data |= PROJECTILEFLAG_FREEZE;
 
 	pProj->m_X = (int)(m_Pos.x * 100.0f);
 	pProj->m_Y = (int)(m_Pos.y * 100.0f);
-	pProj->m_VelX = (int)(Angle * 1000000.0f);
-	pProj->m_VelY = Data;
+	pProj->m_Angle = (int)(Angle * 1000000.0f);
+	pProj->m_Data = Data;
 	pProj->m_StartTick = m_StartTick;
 	pProj->m_Type = m_Type;
+	return true;
 }
