@@ -17,7 +17,7 @@ CSaveTee::~CSaveTee()
 {
 }
 
-void CSaveTee::save(CCharacter *pChr)
+void CSaveTee::Save(CCharacter *pChr)
 {
 	m_ClientID = pChr->m_pPlayer->GetCID();
 	str_copy(m_aName, pChr->Server()->ClientName(m_ClientID), sizeof(m_aName));
@@ -55,7 +55,7 @@ void CSaveTee::save(CCharacter *pChr)
 	m_TuneZoneOld = pChr->m_TuneZoneOld;
 
 	if(pChr->m_StartTime)
-		m_Time = pChr->Server()->Tick() - pChr->m_StartTime;
+		m_Time = pChr->Server()->Tick() - pChr->m_StartTime + g_Config.m_SvSaveSwapGamesPenalty * pChr->Server()->TickSpeed();
 	else
 		m_Time = 0;
 
@@ -108,7 +108,7 @@ void CSaveTee::save(CCharacter *pChr)
 	FormatUuid(pChr->GameServer()->GameUuid(), m_aGameUuid, sizeof(m_aGameUuid));
 }
 
-void CSaveTee::load(CCharacter *pChr, int Team)
+void CSaveTee::Load(CCharacter *pChr, int Team)
 {
 	pChr->m_pPlayer->Pause(m_Paused, true);
 
@@ -209,9 +209,6 @@ void CSaveTee::load(CCharacter *pChr, int Team)
 
 char *CSaveTee::GetString(const CSaveTeam *pTeam)
 {
-	// Add time penalty of 60 seconds (only to the database)
-	int Time = m_Time + 60 * SERVER_TICK_SPEED;
-
 	int HookedPlayer = -1;
 	if(m_HookedPlayer != -1)
 	{
@@ -268,7 +265,7 @@ char *CSaveTee::GetString(const CSaveTeam *pTeam)
 		m_LastWeapon, m_QueuedWeapon,
 		// tee states
 		m_SuperJump, m_Jetpack, m_NinjaJetpack, m_FreezeTime, m_FreezeTick, m_DeepFreeze, m_EndlessHook,
-		m_DDRaceState, m_Hit, m_Collision, m_TuneZone, m_TuneZoneOld, m_Hook, Time,
+		m_DDRaceState, m_Hit, m_Collision, m_TuneZone, m_TuneZoneOld, m_Hook, m_Time,
 		(int)m_Pos.x, (int)m_Pos.y, (int)m_PrevPos.x, (int)m_PrevPos.y,
 		m_TeleCheckpoint, m_LastPenalty,
 		(int)m_CorePos.x, (int)m_CorePos.y, m_Vel.x, m_Vel.y,
@@ -415,7 +412,7 @@ CSaveTeam::~CSaveTeam()
 		delete[] m_pSavedTees;
 }
 
-int CSaveTeam::save(int Team)
+int CSaveTeam::Save(int Team)
 {
 	if(g_Config.m_SvTeam == 3 || (Team > 0 && Team < MAX_CLIENTS))
 	{
@@ -447,7 +444,7 @@ int CSaveTeam::save(int Team)
 				continue;
 			if(m_MembersCount == j)
 				return 3;
-			m_pSavedTees[j].save(p);
+			m_pSavedTees[j].Save(p);
 			j++;
 		}
 		if(m_MembersCount != j)
@@ -498,7 +495,7 @@ bool CSaveTeam::HandleSaveError(int Result, int ClientID, CGameContext *pGameCon
 	return true;
 }
 
-void CSaveTeam::load(int Team, bool KeepCurrentWeakStrong)
+void CSaveTeam::Load(int Team, bool KeepCurrentWeakStrong)
 {
 	CGameTeams *pTeams = &(((CGameControllerDDRace *)m_pController)->m_Teams);
 
@@ -513,7 +510,7 @@ void CSaveTeam::load(int Team, bool KeepCurrentWeakStrong)
 		if(m_pController->GameServer()->m_apPlayers[ClientID] && pTeams->m_Core.Team(ClientID) == Team)
 		{
 			CCharacter *pChr = MatchCharacter(m_pSavedTees[i].GetClientID(), i, KeepCurrentWeakStrong);
-			m_pSavedTees[i].load(pChr, Team);
+			m_pSavedTees[i].Load(pChr, Team);
 		}
 	}
 
