@@ -19,12 +19,31 @@ void CGameTeams::Reset()
 	for(int i = 0; i < MAX_CLIENTS; ++i)
 	{
 		m_TeamState[i] = TEAMSTATE_EMPTY;
+		m_TeamLocked[i] = false;
 		m_TeeFinished[i] = false;
 		m_LastChat[i] = 0;
-		m_TeamLocked[i] = false;
+		m_pSaveTeamResult[i] = nullptr;
+
 		m_Invited[i] = 0;
 		m_Practice[i] = false;
-		m_pSaveTeamResult[i] = nullptr;
+		m_LastSwap[i] = 0;
+	}
+}
+
+void CGameTeams::ResetRoundState(int Team)
+{
+	ResetInvited(Team);
+	ResetSwitchers(Team);
+	m_LastSwap[Team] = 0;
+
+	m_Practice[Team] = 0;
+	for(int i = 0; i < MAX_CLIENTS; i++)
+	{
+		if(m_Core.Team(i) == Team && GameServer()->m_apPlayers[i])
+		{
+			GameServer()->m_apPlayers[i]->m_VotedForPractice = false;
+			GameServer()->m_apPlayers[i]->m_SwapTargetsClientID = -1;
+		}
 	}
 }
 
@@ -268,13 +287,23 @@ const char *CGameTeams::SetCharacterTeam(int ClientID, int Team)
 
 void CGameTeams::SetForceCharacterTeam(int ClientID, int Team)
 {
-	if(Team != m_Core.Team(ClientID))
-		ForceLeaveTeam(ClientID);
-	else
-		m_TeeFinished[ClientID] = false;
-
+	m_TeeFinished[ClientID] = false;
 	int OldTeam = m_Core.Team(ClientID);
 
+	if(Team != OldTeam && (OldTeam != TEAM_FLOCK || g_Config.m_SvTeam == 3) && OldTeam != TEAM_SUPER && m_TeamState[OldTeam] != TEAMSTATE_EMPTY)
+	{
+		bool NoElseInOldTeam = Count(OldTeam) <= 1;
+		if(NoElseInOldTeam)
+		{
+			m_TeamState[OldTeam] = TEAMSTATE_EMPTY;
+
+			// unlock team when last player leaves
+			SetTeamLock(OldTeam, false);
+			ResetRoundState(OldTeam);
+			// do not reset SaveTeamResult, because it should be logged into teehistorian even if the team leaves
+		}
+	}
+
 	m_Core.Team(ClientID, Team);
 
 	if(OldTeam != Team)
@@ -284,7 +313,10 @@ void CGameTeams::SetForceCharacterTeam(int ClientID, int Team)
 				SendTeamsState(LoopClientID);
 
 		if(GetPlayer(ClientID))
+		{
 			GetPlayer(ClientID)->m_VotedForPractice = false;
+			GetPlayer(ClientID)->m_SwapTargetsClientID = -1;
+		}
 	}
 
 	if(Team != TEAM_SUPER && (m_TeamState[Team] == TEAMSTATE_EMPTY || m_TeamLocked[Team]))
@@ -296,32 +328,6 @@ void CGameTeams::SetForceCharacterTeam(int ClientID, int Team)
 	}
 }
 
-void CGameTeams::ForceLeaveTeam(int ClientID)
-{
-	m_TeeFinished[ClientID] = false;
-
-	if((m_Core.Team(ClientID) != TEAM_FLOCK || g_Config.m_SvTeam == 3) && m_Core.Team(ClientID) != TEAM_SUPER && m_TeamState[m_Core.Team(ClientID)] != TEAMSTATE_EMPTY)
-	{
-		bool NoOneInOldTeam = true;
-		for(int i = 0; i < MAX_CLIENTS; ++i)
-			if(i != ClientID && m_Core.Team(ClientID) == m_Core.Team(i))
-			{
-				NoOneInOldTeam = false; // all good exists someone in old team
-				break;
-			}
-		if(NoOneInOldTeam)
-		{
-			m_TeamState[m_Core.Team(ClientID)] = TEAMSTATE_EMPTY;
-
-			// unlock team when last player leaves
-			SetTeamLock(m_Core.Team(ClientID), false);
-			ResetInvited(m_Core.Team(ClientID));
-			m_Practice[m_Core.Team(ClientID)] = false;
-			// do not reset SaveTeamResult, because it should be logged into teehistorian even if the team leaves
-		}
-	}
-}
-
 int CGameTeams::Count(int Team) const
 {
 	if(Team == TEAM_SUPER)
@@ -423,15 +429,24 @@ void CGameTeams::SendTeamsState(int ClientID)
 	if(g_Config.m_SvTeam == 3)
 		return;
 
-	if(!m_pGameContext->m_apPlayers[ClientID] || m_pGameContext->m_apPlayers[ClientID]->GetClientVersion() <= VERSION_DDRACE)
+	if(!m_pGameContext->m_apPlayers[ClientID])
 		return;
 
 	CMsgPacker Msg(NETMSGTYPE_SV_TEAMSSTATE);
+	CMsgPacker MsgLegacy(NETMSGTYPE_SV_TEAMSSTATELEGACY);
 
 	for(unsigned i = 0; i < MAX_CLIENTS; i++)
+	{
 		Msg.AddInt(m_Core.Team(i));
+		MsgLegacy.AddInt(m_Core.Team(i));
+	}
 
 	Server()->SendMsg(&Msg, MSGFLAG_VITAL, ClientID);
+	int ClientVersion = m_pGameContext->m_apPlayers[ClientID]->GetClientVersion();
+	if(!Server()->IsSixup(ClientID) && VERSION_DDRACE < ClientVersion && ClientVersion < VERSION_DDNET_MSG_LEGACY)
+	{
+		Server()->SendMsg(&MsgLegacy, MSGFLAG_VITAL, ClientID);
+	}
 }
 
 int CGameTeams::GetDDRaceState(CPlayer *Player)
@@ -628,20 +643,24 @@ void CGameTeams::OnFinish(CPlayer *Player, float Time, const char *pTimestamp)
 		}
 	}
 
-	if(Player->GetClientVersion() >= VERSION_DDRACE)
 	{
 		CNetMsg_Sv_DDRaceTime Msg;
-		Msg.m_Time = (int)(Time * 100.0f);
-		Msg.m_Check = 0;
-		Msg.m_Finish = 1;
+		CNetMsg_Sv_DDRaceTimeLegacy MsgLegacy;
+		MsgLegacy.m_Time = Msg.m_Time = (int)(Time * 100.0f);
+		MsgLegacy.m_Check = Msg.m_Check = 0;
+		MsgLegacy.m_Finish = Msg.m_Finish = 1;
 
 		if(pData->m_BestTime)
 		{
 			float Diff = (Time - pData->m_BestTime) * 100;
-			Msg.m_Check = (int)Diff;
+			MsgLegacy.m_Check = Msg.m_Check = (int)Diff;
 		}
 
 		Server()->SendPackMsg(&Msg, MSGFLAG_VITAL, ClientID);
+		if(!Server()->IsSixup(ClientID) && VERSION_DDRACE <= Player->GetClientVersion() && Player->GetClientVersion() < VERSION_DDNET_MSG_LEGACY)
+		{
+			Server()->SendPackMsg(&MsgLegacy, MSGFLAG_VITAL, ClientID);
+		}
 	}
 
 	{
@@ -662,6 +681,96 @@ void CGameTeams::OnFinish(CPlayer *Player, float Time, const char *pTimestamp)
 	}
 }
 
+void CGameTeams::RequestTeamSwap(CPlayer *pPlayer, CPlayer *pTargetPlayer, int Team)
+{
+	if(!pPlayer || !pTargetPlayer)
+		return;
+
+	char aBuf[512];
+	if(pPlayer->m_SwapTargetsClientID == pTargetPlayer->GetCID())
+	{
+		str_format(aBuf, sizeof(aBuf),
+			"%s has already requested to swap with %s.",
+			Server()->ClientName(pPlayer->GetCID()), Server()->ClientName(pTargetPlayer->GetCID()));
+
+		GameServer()->SendChatTeam(Team, aBuf);
+		return;
+	}
+
+	str_format(aBuf, sizeof(aBuf),
+		"%s has requested to swap with %s. Please wait %d seconds then type /swap %s.",
+		Server()->ClientName(pPlayer->GetCID()), Server()->ClientName(pTargetPlayer->GetCID()), g_Config.m_SvSaveSwapGamesDelay, Server()->ClientName(pPlayer->GetCID()));
+
+	GameServer()->SendChatTeam(Team, aBuf);
+
+	pPlayer->m_SwapTargetsClientID = pTargetPlayer->GetCID();
+	m_LastSwap[Team] = Server()->Tick();
+}
+
+void CGameTeams::SwapTeamCharacters(CPlayer *pPlayer, CPlayer *pTargetPlayer, int Team)
+{
+	if(!pPlayer || !pTargetPlayer)
+		return;
+
+	char aBuf[128];
+
+	int Since = (Server()->Tick() - m_LastSwap[Team]) / Server()->TickSpeed();
+	if(Since < g_Config.m_SvSaveSwapGamesDelay)
+	{
+		str_format(aBuf, sizeof(aBuf),
+			"You have to wait %d seconds until you can swap.",
+			g_Config.m_SvSaveSwapGamesDelay - Since);
+
+		GameServer()->SendChatTeam(Team, aBuf);
+
+		return;
+	}
+
+	for(int i = 0; i < MAX_CLIENTS; i++)
+	{
+		if(m_Core.Team(i) == Team && GameServer()->m_apPlayers[i])
+		{
+			GameServer()->m_apPlayers[i]->m_SwapTargetsClientID = -1;
+		}
+	}
+
+	int TimeoutAfterDelay = g_Config.m_SvSaveSwapGamesDelay + g_Config.m_SvSwapTimeout;
+	if(Since >= TimeoutAfterDelay)
+	{
+		str_format(aBuf, sizeof(aBuf),
+			"Your swap request timed out %d seconds ago. Use /swap again to re-initiate it.",
+			Since - g_Config.m_SvSwapTimeout);
+
+		GameServer()->SendChatTeam(Team, aBuf);
+
+		return;
+	}
+
+	for(int i = 0; i < MAX_CLIENTS; i++)
+	{
+		if(m_Core.Team(i) == Team && GameServer()->m_apPlayers[i])
+		{
+			GameServer()->m_apPlayers[i]->GetCharacter()->ResetHook();
+			GameServer()->m_World.ReleaseHooked(i);
+		}
+	}
+
+	CSaveTee PrimarySavedTee;
+	PrimarySavedTee.Save(pPlayer->GetCharacter());
+
+	CSaveTee SecondarySavedTee;
+	SecondarySavedTee.Save(pTargetPlayer->GetCharacter());
+
+	PrimarySavedTee.Load(pTargetPlayer->GetCharacter(), Team);
+	SecondarySavedTee.Load(pPlayer->GetCharacter(), Team);
+
+	str_format(aBuf, sizeof(aBuf),
+		"%s has swapped with %s.",
+		Server()->ClientName(pPlayer->GetCID()), Server()->ClientName(pTargetPlayer->GetCID()));
+
+	GameServer()->SendChatTeam(Team, aBuf);
+}
+
 void CGameTeams::ProcessSaveTeam()
 {
 	for(int Team = 0; Team < MAX_CLIENTS; Team++)
@@ -704,7 +813,7 @@ void CGameTeams::ProcessSaveTeam()
 			if(Count(Team) > 0)
 			{
 				// load weak/strong order to prevent switching weak/strong while saving
-				m_pSaveTeamResult[Team]->m_SavedTeam.load(Team, false);
+				m_pSaveTeamResult[Team]->m_SavedTeam.Load(Team, false);
 			}
 			break;
 		case CScoreSaveResult::LOAD_SUCCESS:
@@ -719,7 +828,7 @@ void CGameTeams::ProcessSaveTeam()
 			if(Count(Team) > 0)
 			{
 				// keep current weak/strong order as on some maps there is no other way of switching
-				m_pSaveTeamResult[Team]->m_SavedTeam.load(Team, true);
+				m_pSaveTeamResult[Team]->m_SavedTeam.Load(Team, true);
 			}
 			char aSaveID[UUID_MAXSTRSIZE];
 			FormatUuid(m_pSaveTeamResult[Team]->m_SaveID, aSaveID, UUID_MAXSTRSIZE);
@@ -764,12 +873,10 @@ void CGameTeams::OnCharacterDeath(int ClientID, int Weapon)
 		return;
 	bool Locked = TeamLocked(Team) && Weapon != WEAPON_GAME;
 
-	if(g_Config.m_SvTeam == 3)
+	if(g_Config.m_SvTeam == 3 && Team != TEAM_SUPER)
 	{
 		ChangeTeamState(Team, CGameTeams::TEAMSTATE_OPEN);
-		ResetSwitchers(Team);
-		m_Practice[Team] = false;
-		GameServer()->m_apPlayers[ClientID]->m_VotedForPractice = false;
+		ResetRoundState(Team);
 	}
 	else if(Locked)
 	{
@@ -846,8 +953,7 @@ void CGameTeams::ResetSavedTeam(int ClientID, int Team)
 	if(g_Config.m_SvTeam == 3)
 	{
 		ChangeTeamState(Team, CGameTeams::TEAMSTATE_OPEN);
-		ResetSwitchers(Team);
-		m_Practice[Team] = false;
+		ResetRoundState(Team);
 	}
 	else
 	{
