@@ -218,6 +218,12 @@ int main(int argc, const char **argv)
 		pItem = g_DataReader.GetItem(Index, &Type, &ID);
 		Size = g_DataReader.GetItemSize(Index);
 
+		// filter ITEMTYPE_EX items, they will be automatically added again
+		if(Type == ITEMTYPE_EX)
+		{
+			continue;
+		}
+
 		Success &= CheckImageDimensions(pItem, Type, pSourceFileName);
 
 		pItem = ReplaceImageItem(pItem, Type, &NewImageItem);
