@@ -28,6 +28,13 @@ int main(int argc, const char **argv)
 	{
 		pPtr = DataFile.GetItem(Index, &Type, &ID);
 		Size = DataFile.GetItemSize(Index);
+
+		// filter ITEMTYPE_EX items, they will be automatically added again
+		if(Type == ITEMTYPE_EX)
+		{
+			continue;
+		}
+
 		df.AddItem(Type, ID, Size, pPtr);
 	}
 
